-module(pgm).

%% postgresql conn and queries wrappers module


-export([
  mypg/0,
  transaction/2,
  select/3,
  in_up_del/3,
  returning/3
]).


mypg() ->
  %{ok, C} = epgsql:connect("localhost", "user", "pass", [{database, "test"}, {port, 5432}, {timeout, 2000}]), %% can be used without connection pool
  Conn = epgsql:connect(
    application:get_env(n4u, pgs_host, "localhost"), %% can be used without connection pool
    application:get_env(n4u, pgs_user, "user"),      %% gets settings from sys.config
    application:get_env(n4u, pgs_pass, "pass"),
    [{database, application:get_env(n4u, pgs_db, "test")},
     {port, application:get_env(n4u, pgs_port, 6432)},
     {timeout, 3000}]),
  case Conn of
    {ok, C} ->
      C;
    {error, M} ->
      io:format("Database error: ~p~n", [epgsql_errcodes:to_name(M)]),
      wf:wire(["alert('db conn error!');"]),
      %{error, M}
      error
  end.


transaction(Ppid, Fun) ->
  case epgsql:squery(Ppid, "BEGIN") of
    {ok, _, _} ->
      try
        Result = Fun(),
        epgsql:squery(Ppid, "COMMIT"),
        Result
      catch
        Err1:Err2 ->
          epgsql:squery(Ppid, "ROLLBACK"),
          {Err1, Err2}
      end;
    Error -> Error
  end.


select(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, _, R} ->
      R;
    {error, E} ->
      io:format("~p~n", [E]),
      {error, E}
  end.


in_up_del(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, R} ->
      R;
    {error, E} ->
      io:format("~p~n", [E]),
      {error, E}
  end.


returning(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, 1, _, R} ->
      R;
    {error, E} ->
      io:format("~p~n", [E]),
      {error, E}
  end.

